@@ -28,6 +28,7 @@ OPERATING_STATUS = 'operating_status'
 POOLS = 'pools'
 PROJECT_ID = 'project_id'
 PROVIDER = 'provider'
+PROVIDER_NAME = 'provider_name'
 PROVISIONING_STATUS = 'provisioning_status'
 REQUEST_ERRORS = 'request_errors'
 TOTAL_CONNECTIONS = 'total_connections'
@@ -77,6 +78,10 @@ HTTP_METHOD = 'http_method'
 URL_PATH = 'url_path'
 EXPECTED_CODES = 'expected_codes'
 
+FLAVOR_DATA = 'flavor_data'
+ENABLED = 'enabled'
+FLAVOR_PROFILE_ID = 'flavor_profile_id'
+
 # Other constants
 ACTIVE = 'ACTIVE'
 ADMIN_STATE_UP_TRUE = 'true'
@@ -89,6 +94,8 @@ ONLINE = 'ONLINE'
 NO_MONITOR = 'NO_MONITOR'
 ERROR = 'ERROR'
 SORT = 'sort'
+SINGLE = 'SINGLE'
+ACTIVE_STANDBY = 'ACTIVE_STANDBY'
 
 # Protocols
 HTTP = 'HTTP'
@@ -185,6 +192,9 @@ AMPHORA_STATUSES = (
     STATUS_PENDING_DELETE, STATUS_DELETED, STATUS_ERROR
 )
 
+# Flavor capabilities
+LOADBALANCER_TOPOLOGY = 'loadbalancer_topology'
+
 # API valid fields
 SHOW_LOAD_BALANCER_RESPONSE_FIELDS = (
     ADMIN_STATE_UP, CREATED_AT, DESCRIPTION, FLAVOR_ID, ID, LISTENERS, NAME,
@@ -232,3 +242,7 @@ SHOW_AMPHORA_RESPONSE_FIELDS = [
     VRRP_PORT_ID, HA_PORT_ID, CERT_EXPIRATION, CERT_BUSY, ROLE, STATUS,
     VRRP_INTERFACE, VRRP_ID, VRRP_PRIORITY, CACHED_ZONE
 ]
+
+SHOW_FLAVOR_PROFILE_FIELDS = [ID, NAME, PROVIDER_NAME, FLAVOR_DATA]
+
+SHOW_FLAVOR_FIELDS = [ID, NAME, DESCRIPTION, ENABLED, FLAVOR_PROFILE_ID]
