@@ -14,9 +14,13 @@
 
 
 from oslo_config import cfg
+from oslo_log import log as logging
 
 from octavia_tempest_plugin.common import constants as const
 
+
+LOG = logging.getLogger(__name__)
+
 service_available_group = cfg.OptGroup(name='service_available',
                                        title='Available OpenStack Services')
 
@@ -54,6 +58,9 @@ OctaviaGroup = [
                default=300,
                help='Timeout in seconds to wait for non-load-balancer '
                     'resources to build'),
+    cfg.StrOpt('octavia_svc_username', default='admin',
+               help='The service_auth username the Octavia services are using'
+                    'to access other OpenStack services.'),
     # load-balancer specific options
     cfg.IntOpt('check_interval',
                default=5,
@@ -94,6 +101,16 @@ OctaviaGroup = [
                help='Type of RBAC tests to run. "advanced" runs the octavia '
                     'default RBAC tests. "owner_or_admin" runs the legacy '
                     'owner or admin tests. "none" disables the RBAC tests.'),
+    cfg.DictOpt('enabled_provider_drivers',
+                help=('List of enabled provider drivers and description '
+                      'dictionaries. Must match the driver name in the '
+                      'octavia.api.drivers entrypoint. Example: '
+                      '{\'amphora\': \'The Octavia Amphora driver.\', '
+                      '\'octavia\': \'Deprecated alias of the Octavia '
+                      'Amphora driver.\'}'),
+                default={'amphora': 'The Octavia Amphora driver.',
+                         'octavia': 'Deprecated alias of the Octavia Amphora '
+                         'driver.'}),
     # Networking
     cfg.BoolOpt('test_with_ipv6',
                 default=True,
@@ -152,4 +169,16 @@ LBFeatureEnabledGroup = [
                 default=True,
                 help="Whether Health Monitor is available with provider"
                      " driver or not."),
+    cfg.BoolOpt('terminated_tls_enabled',
+                default=True,
+                help="Whether TLS termination is available with provider "
+                     "driver or not."),
+    cfg.BoolOpt('l7_protocol_enabled',
+                default=True,
+                help="Whether L7 Protocols are available with the provider"
+                     " driver or not."),
+    cfg.StrOpt('l4_protocol',
+               default="TCP",
+               help="The type of L4 Protocol which is supported with the"
+                    " provider driver."),
 ]
