@@ -33,6 +33,12 @@ LOG = logging.getLogger(__name__)
 class HealthMonitorAPITest(test_base.LoadBalancerBaseTest):
     """Test the healthmonitor object API."""
 
+    @classmethod
+    def skip_checks(cls):
+        super(HealthMonitorAPITest, cls).skip_checks()
+        if not CONF.loadbalancer_feature_enabled.health_monitor_enabled:
+            raise cls.skipException('Health Monitors not supported')
+
     @classmethod
     def resource_setup(cls):
         """Setup resources needed by the tests."""
