@@ -30,6 +30,7 @@ from tempest.lib.common.utils import data_utils
 from tempest.lib.common.utils.linux import remote_client
 from tempest.lib import exceptions
 from tempest import test
+import tenacity
 
 from octavia_tempest_plugin import clients
 from octavia_tempest_plugin.common import constants as const
@@ -39,6 +40,11 @@ from octavia_tempest_plugin.tests import waiters
 CONF = config.CONF
 LOG = logging.getLogger(__name__)
 
+RETRY_ATTEMPTS = 15
+RETRY_INITIAL_DELAY = 1
+RETRY_BACKOFF = 1
+RETRY_MAX = 5
+
 
 class LoadBalancerBaseTest(test.BaseTestCase):
     """Base class for load balancer tests."""
@@ -75,7 +81,7 @@ class LoadBalancerBaseTest(test.BaseTestCase):
 
         for service, available in service_list.items():
             if not available:
-                skip_msg = ("{0} skipped as {1} serivce is not "
+                skip_msg = ("{0} skipped as {1} service is not "
                             "available.".format(cls.__name__, service))
                 raise cls.skipException(skip_msg)
 
@@ -116,6 +122,11 @@ class LoadBalancerBaseTest(test.BaseTestCase):
         cls.mem_l7policy_client = cls.os_roles_lb_member.l7policy_client
         cls.mem_l7rule_client = cls.os_roles_lb_member.l7rule_client
         cls.mem_amphora_client = cls.os_roles_lb_member.amphora_client
+        cls.lb_admin_flavor_profile_client = (
+            cls.os_roles_lb_admin.flavor_profile_client)
+        cls.lb_admin_flavor_client = cls.os_roles_lb_admin.flavor_client
+        cls.mem_flavor_client = cls.os_roles_lb_member.flavor_client
+        cls.mem_provider_client = cls.os_roles_lb_member.provider_client
 
     @classmethod
     def resource_setup(cls):
@@ -140,6 +151,7 @@ class LoadBalancerBaseTest(test.BaseTestCase):
             cls.lb_member_2_net = {'id': uuidutils.generate_uuid()}
             cls.lb_member_2_subnet = {'id': uuidutils.generate_uuid()}
             if CONF.load_balancer.test_with_ipv6:
+                cls.lb_member_vip_ipv6_net = {'id': uuidutils.generate_uuid()}
                 cls.lb_member_vip_ipv6_subnet = {'id':
                                                  uuidutils.generate_uuid()}
                 cls.lb_member_1_ipv6_subnet = {'id': uuidutils.generate_uuid()}
@@ -202,6 +214,42 @@ class LoadBalancerBaseTest(test.BaseTestCase):
                 LOG.debug('Octavia Setup: lb_member_2_ipv6_subnet = {}'.format(
                     cls.lb_member_2_ipv6_subnet[const.ID]))
 
+    @classmethod
+    # Neutron can be slow to clean up ports from the subnets/networks.
+    # Retry this delete a few times if we get a "Conflict" error to give
+    # neutron time to fully cleanup the ports.
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(exceptions.Conflict),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
+    def _logging_delete_network(cls, net_id):
+        try:
+            cls.lb_mem_net_client.delete_network(net_id)
+        except Exception:
+            LOG.error('Unable to delete network {}. Active ports:'.format(
+                net_id))
+            LOG.error(cls.lb_mem_ports_client.list_ports())
+            raise
+
+    @classmethod
+    # Neutron can be slow to clean up ports from the subnets/networks.
+    # Retry this delete a few times if we get a "Conflict" error to give
+    # neutron time to fully cleanup the ports.
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(exceptions.Conflict),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
+    def _logging_delete_subnet(cls, subnet_id):
+        try:
+            cls.lb_mem_subnet_client.delete_subnet(subnet_id)
+        except Exception:
+            LOG.error('Unable to delete subnet {}. Active ports:'.format(
+                subnet_id))
+            LOG.error(cls.lb_mem_ports_client.list_ports())
+            raise
+
     @classmethod
     def _create_networks(cls):
         """Creates networks, subnets, and routers used in tests.
@@ -229,7 +277,7 @@ class LoadBalancerBaseTest(test.BaseTestCase):
         LOG.info('lb_member_vip_net: {}'.format(cls.lb_member_vip_net))
         cls.addClassResourceCleanup(
             waiters.wait_for_not_found,
-            cls.lb_mem_net_client.delete_network,
+            cls._logging_delete_network,
             cls.lb_mem_net_client.show_network,
             cls.lb_member_vip_net['id'])
 
@@ -244,26 +292,36 @@ class LoadBalancerBaseTest(test.BaseTestCase):
         LOG.info('lb_member_vip_subnet: {}'.format(cls.lb_member_vip_subnet))
         cls.addClassResourceCleanup(
             waiters.wait_for_not_found,
-            cls.lb_mem_subnet_client.delete_subnet,
+            cls._logging_delete_subnet,
             cls.lb_mem_subnet_client.show_subnet,
             cls.lb_member_vip_subnet['id'])
 
         # Create tenant VIP IPv6 subnet
         if CONF.load_balancer.test_with_ipv6:
-            subnet_kwargs = {
-                'name': data_utils.rand_name("lb_member_vip_ipv6_subnet"),
-                'network_id': cls.lb_member_vip_net['id'],
-                'cidr': CONF.load_balancer.vip_ipv6_subnet_cidr,
-                'ip_version': 6}
-            result = cls.lb_mem_subnet_client.create_subnet(**subnet_kwargs)
-            cls.lb_member_vip_ipv6_subnet = result['subnet']
+            # See if ipv6-public-subnet exists and use it if so.
+            pub_ipv6_subnet = cls.os_admin.subnets_client.list_subnets(
+                name='ipv6-public-subnet')['subnets']
+
+            if len(pub_ipv6_subnet) == 1:
+                cls.lb_member_vip_ipv6_subnet = pub_ipv6_subnet[0]
+                cls.lb_member_vip_ipv6_net = {
+                    'id': pub_ipv6_subnet[0]['network_id']}
+            else:
+                subnet_kwargs = {
+                    'name': data_utils.rand_name("lb_member_vip_ipv6_subnet"),
+                    'network_id': cls.lb_member_vip_net['id'],
+                    'cidr': CONF.load_balancer.vip_ipv6_subnet_cidr,
+                    'ip_version': 6}
+                result = cls.lb_mem_subnet_client.create_subnet(
+                    **subnet_kwargs)
+                cls.lb_member_vip_ipv6_subnet = result['subnet']
+                cls.addClassResourceCleanup(
+                    waiters.wait_for_not_found,
+                    cls._logging_delete_subnet,
+                    cls.lb_mem_subnet_client.show_subnet,
+                    cls.lb_member_vip_ipv6_subnet['id'])
             LOG.info('lb_member_vip_ipv6_subnet: {}'.format(
                 cls.lb_member_vip_ipv6_subnet))
-            cls.addClassResourceCleanup(
-                waiters.wait_for_not_found,
-                cls.lb_mem_subnet_client.delete_subnet,
-                cls.lb_mem_subnet_client.show_subnet,
-                cls.lb_member_vip_ipv6_subnet['id'])
 
         # Create tenant member 1 network
         network_kwargs = {
@@ -278,7 +336,7 @@ class LoadBalancerBaseTest(test.BaseTestCase):
         LOG.info('lb_member_1_net: {}'.format(cls.lb_member_1_net))
         cls.addClassResourceCleanup(
             waiters.wait_for_not_found,
-            cls.lb_mem_net_client.delete_network,
+            cls._logging_delete_network,
             cls.lb_mem_net_client.show_network,
             cls.lb_member_1_net['id'])
 
@@ -293,7 +351,7 @@ class LoadBalancerBaseTest(test.BaseTestCase):
         LOG.info('lb_member_1_subnet: {}'.format(cls.lb_member_1_subnet))
         cls.addClassResourceCleanup(
             waiters.wait_for_not_found,
-            cls.lb_mem_subnet_client.delete_subnet,
+            cls._logging_delete_subnet,
             cls.lb_mem_subnet_client.show_subnet,
             cls.lb_member_1_subnet['id'])
 
@@ -305,12 +363,16 @@ class LoadBalancerBaseTest(test.BaseTestCase):
                 'cidr': CONF.load_balancer.member_1_ipv6_subnet_cidr,
                 'ip_version': 6}
             result = cls.lb_mem_subnet_client.create_subnet(**subnet_kwargs)
+            cls.lb_member_1_subnet_prefix = (
+                CONF.load_balancer.member_1_ipv6_subnet_cidr.rpartition('/')[2]
+                )
+            assert(cls.lb_member_1_subnet_prefix.isdigit())
             cls.lb_member_1_ipv6_subnet = result['subnet']
             LOG.info('lb_member_1_ipv6_subnet: {}'.format(
                 cls.lb_member_1_ipv6_subnet))
             cls.addClassResourceCleanup(
                 waiters.wait_for_not_found,
-                cls.lb_mem_subnet_client.delete_subnet,
+                cls._logging_delete_subnet,
                 cls.lb_mem_subnet_client.show_subnet,
                 cls.lb_member_1_ipv6_subnet['id'])
 
@@ -327,7 +389,7 @@ class LoadBalancerBaseTest(test.BaseTestCase):
         LOG.info('lb_member_2_net: {}'.format(cls.lb_member_2_net))
         cls.addClassResourceCleanup(
             waiters.wait_for_not_found,
-            cls.lb_mem_net_client.delete_network,
+            cls._logging_delete_network,
             cls.lb_mem_net_client.show_network,
             cls.lb_member_2_net['id'])
 
@@ -342,7 +404,7 @@ class LoadBalancerBaseTest(test.BaseTestCase):
         LOG.info('lb_member_2_subnet: {}'.format(cls.lb_member_2_subnet))
         cls.addClassResourceCleanup(
             waiters.wait_for_not_found,
-            cls.lb_mem_subnet_client.delete_subnet,
+            cls._logging_delete_subnet,
             cls.lb_mem_subnet_client.show_subnet,
             cls.lb_member_2_subnet['id'])
 
@@ -354,12 +416,16 @@ class LoadBalancerBaseTest(test.BaseTestCase):
                 'cidr': CONF.load_balancer.member_2_ipv6_subnet_cidr,
                 'ip_version': 6}
             result = cls.lb_mem_subnet_client.create_subnet(**subnet_kwargs)
+            cls.lb_member_2_subnet_prefix = (
+                CONF.load_balancer.member_2_ipv6_subnet_cidr.rpartition('/')[2]
+                )
+            assert(cls.lb_member_2_subnet_prefix.isdigit())
             cls.lb_member_2_ipv6_subnet = result['subnet']
             LOG.info('lb_member_2_ipv6_subnet: {}'.format(
                 cls.lb_member_2_ipv6_subnet))
             cls.addClassResourceCleanup(
                 waiters.wait_for_not_found,
-                cls.lb_mem_subnet_client.delete_subnet,
+                cls._logging_delete_subnet,
                 cls.lb_mem_subnet_client.show_subnet,
                 cls.lb_member_2_ipv6_subnet['id'])
 
@@ -368,21 +434,27 @@ class LoadBalancerBaseTest(test.BaseTestCase):
                                  use_fixed_ip=False):
         if not ip_version:
             ip_version = 6 if CONF.load_balancer.test_with_ipv6 else 4
-        if cls.lb_member_vip_subnet:
+        if cls.lb_member_vip_subnet or cls.lb_member_vip_ipv6_subnet:
             ip_index = data_utils.rand_int_id(start=10, end=100)
             while ip_index in cls.used_ips:
                 ip_index = data_utils.rand_int_id(start=10, end=100)
             cls.used_ips.append(ip_index)
             if ip_version == 4:
-                network = ipaddress.IPv4Network(
-                    six.u(CONF.load_balancer.vip_subnet_cidr))
-                lb_vip_address = str(network[ip_index])
                 subnet_id = cls.lb_member_vip_subnet[const.ID]
+                if CONF.load_balancer.test_with_noop:
+                    lb_vip_address = '198.18.33.33'
+                else:
+                    subnet = cls.os_admin.subnets_client.show_subnet(subnet_id)
+                    network = ipaddress.IPv4Network(subnet['subnet']['cidr'])
+                    lb_vip_address = str(network[ip_index])
             else:
-                network = ipaddress.IPv6Network(
-                    six.u(CONF.load_balancer.vip_ipv6_subnet_cidr))
-                lb_vip_address = str(network[ip_index])
                 subnet_id = cls.lb_member_vip_ipv6_subnet[const.ID]
+                if CONF.load_balancer.test_with_noop:
+                    lb_vip_address = '2001:db8:33:33:33:33:33:33'
+                else:
+                    subnet = cls.os_admin.subnets_client.show_subnet(subnet_id)
+                    network = ipaddress.IPv6Network(subnet['subnet']['cidr'])
+                    lb_vip_address = str(network[ip_index])
             lb_kwargs[const.VIP_SUBNET_ID] = subnet_id
             if use_fixed_ip:
                 lb_kwargs[const.VIP_ADDRESS] = lb_vip_address
@@ -520,6 +592,17 @@ class LoadBalancerBaseTestWithCompute(LoadBalancerBaseTest):
         LOG.debug('Octavia Setup: webserver2_public_ip = {}'.format(
             cls.webserver2_public_ip))
 
+        if CONF.load_balancer.test_with_ipv6:
+            # Enable the IPv6 nic in webserver 1
+            cls._enable_ipv6_nic_webserver(
+                cls.webserver1_public_ip, cls.lb_member_keypair['private_key'],
+                cls.webserver1_ipv6, cls.lb_member_1_subnet_prefix)
+
+            # Enable the IPv6 nic in webserver 2
+            cls._enable_ipv6_nic_webserver(
+                cls.webserver2_public_ip, cls.lb_member_keypair['private_key'],
+                cls.webserver2_ipv6, cls.lb_member_2_subnet_prefix)
+
         # Set up serving on webserver 1
         cls._install_start_webserver(cls.webserver1_public_ip,
                                      cls.lb_member_keypair['private_key'],
@@ -710,6 +793,19 @@ class LoadBalancerBaseTestWithCompute(LoadBalancerBaseTest):
         linux_client.exec_command('sudo screen -d -m {0} -port 81 '
                                   '-id {1}'.format(dest_file, start_id + 1))
 
+    # Cirros does not configure the assigned IPv6 address by default
+    # so enable it manually like tempest does here:
+    # tempest/scenario/test_netowrk_v6.py turn_nic6_on()
+    @classmethod
+    def _enable_ipv6_nic_webserver(cls, ip_address, ssh_key,
+                                   ipv6_address, ipv6_prefix):
+        linux_client = remote_client.RemoteClient(
+            ip_address, CONF.validation.image_ssh_user, pkey=ssh_key)
+        linux_client.validate_authentication()
+
+        linux_client.exec_command('sudo ip address add {0}/{1} dev '
+                                  'eth0'.format(ipv6_address, ipv6_prefix))
+
     @classmethod
     def _validate_webserver(cls, ip_address, start_id):
         URL = 'http://{0}'.format(ip_address)
@@ -717,13 +813,15 @@ class LoadBalancerBaseTestWithCompute(LoadBalancerBaseTest):
         URL = 'http://{0}:81'.format(ip_address)
         validators.validate_URL_response(URL, expected_body=str(start_id + 1))
 
-    def _wait_for_lb_functional(self, vip_address):
+    def _wait_for_lb_functional(self, vip_address,
+                                protocol='http', verify=True):
         session = requests.Session()
         start = time.time()
 
         while time.time() - start < CONF.load_balancer.build_timeout:
             try:
-                session.get("http://{0}".format(vip_address), timeout=2)
+                session.get("{0}://{1}".format(protocol, vip_address),
+                            timeout=2, verify=verify)
                 time.sleep(1)
                 return
             except Exception:
@@ -733,17 +831,21 @@ class LoadBalancerBaseTestWithCompute(LoadBalancerBaseTest):
                   'period. Failing test.')
         raise Exception()
 
-    def check_members_balanced(self, vip_address, traffic_member_count=2):
+    def check_members_balanced(self, vip_address, traffic_member_count=2,
+                               protocol='http', verify=True):
         session = requests.Session()
         response_counts = {}
 
-        self._wait_for_lb_functional(vip_address)
+        if ipaddress.ip_address(vip_address).version == 6:
+            vip_address = '[{}]'.format(vip_address)
+
+        self._wait_for_lb_functional(vip_address, protocol, verify)
 
         # Send a number requests to lb vip
         for i in range(20):
             try:
-                r = session.get('http://{0}'.format(vip_address),
-                                timeout=2)
+                r = session.get('{0}://{1}'.format(protocol, vip_address),
+                                timeout=2, verify=verify)
 
                 if r.content in response_counts:
                     response_counts[r.content] += 1
@@ -774,6 +876,8 @@ class LoadBalancerBaseTestWithCompute(LoadBalancerBaseTest):
         :param cookies: Optional cookies to send in the request.
         :param redirect: Is the request a redirect? If true, assume the passed
                          content should be the next URL in the chain.
+        :param timeout: Optional seconds to wait for the server to send data.
+
         :return: boolean success status
 
         :raises: testtools.matchers.MismatchError
@@ -788,6 +892,7 @@ class LoadBalancerBaseTestWithCompute(LoadBalancerBaseTest):
                 self.assertEqual(response_code, req.status_code)
             if redirect:
                 self.assertTrue(req.is_redirect)
-                self.assertEqual(response_content, req.next.url)
+                self.assertEqual(response_content,
+                                 session.get_redirect_target(req))
             elif response_content:
                 self.assertEqual(six.text_type(response_content), req.text)
